import os
import configparser
import click
import logging
from .constants import APP_NAME

from .commands.configure import configure
from .commands.get_macros import get_macros
from .commands.get_triggers import get_triggers
from .commands.get_automations import get_automations
from .commands.update_macros import update_macros
from .commands.add_macros import add_macros
from .commands.backup_guide import backup_guide
from .commands.create_article_mapping import create_article_mapping
from .commands.upload_theme import upload_theme
from .commands.show_brands import show_brands


@click.group()
@click.option('--profile', type=click.STRING, default='default')
@click.option('--debug', is_flag=True)
@click.pass_context
def cli(ctx, profile, debug):
    if debug:
        try:
            from http.client import HTTPConnection
        except ImportError:
            from httplib import HTTPConnection
        HTTPConnection.debuglevel = 1
        logging.basicConfig()
        logging.getLogger().setLevel(logging.DEBUG)
        requests_log = logging.getLogger('requests.packages.urllib3')
        requests_log.setLevel(logging.DEBUG)
        requests_log.propagate = True

    if ctx.obj is None:
        ctx.obj = {}

    ctx.obj['profile'] = profile
    ctx.obj['configuration'] = {}

    conf_path = os.path.join(click.get_app_dir(APP_NAME), 'config.ini')
    config = configparser.ConfigParser()
    config.read([conf_path])

    if profile in config:
        for key in config[profile]:
            ctx.obj['configuration'][key] = config[profile][key]


cli.add_command(configure)
cli.add_command(get_macros)
cli.add_command(get_triggers)
cli.add_command(get_automations)
cli.add_command(update_macros)
cli.add_command(add_macros)
cli.add_command(backup_guide)
cli.add_command(create_article_mapping)
cli.add_command(upload_theme)
cli.add_command(show_brands)



import time
from functools import reduce
import click
from ..utilities import post_theme_import_job, post_theme, get_theme_job


@click.command()
@click.option('--brand-id', type=click.STRING, required=True, prompt=True)
@click.option('--file', type=click.File(mode='rb'), required=True, prompt=True)
@click.pass_context
def upload_theme(ctx, brand_id, file):
    """Upload help center theme zip file."""
    if ctx.obj['configuration'] == {}:
        raise click.UsageError('No configuration found. Try `zenkly configure`', ctx=ctx)

    click.confirm('Are you sure you want to upload this theme file [\033[36m%s\033[39m]?' % file.name, abort=True)

    job = post_theme_import_job(config=ctx.obj['configuration'], brand_id=brand_id)

    files = {'file': file}

    post_theme(config=ctx.obj['configuration'],
               storage_url=job['data']['upload']['url'],
               parameters=job['data']['upload']['parameters'],
               files=files)

    job_status = 'pending'
    click.secho('Waiting for upload job to complete...')

    while job_status == 'pending':
        time.sleep(1)
        job = get_theme_job(config=ctx.obj['configuration'], job_id=job['id'])
        job_status = job['status']

        if job_status == 'completed':
            click.secho('Complete!')

        if job_status == 'failed':
            errors = reduce(lambda x, y: x + f"\n  {y['title']}: {y['code']}", job['errors'], '  ')
            raise click.ClickException(f"The theme job failed: {click.style(errors, fg='red')}")



import click
from tabulate import tabulate
from ..utilities import get_all_brands


@click.command()
@click.pass_context
def show_brands(ctx):
    """Show brands as tabular data."""
    if ctx.obj['configuration'] == {}:
        raise click.UsageError('No configuration found. Try `zenkly configure`', ctx=ctx)

    brands = get_all_brands(config=ctx.obj['configuration'])

    headers = ['Name', 'ID', 'Subdomain', 'Brand URL', 'HC State', 'Active?', 'Host Mapping']
    table = []
    for brand in brands:
        table.append([
            brand['name'],
            brand['id'],
            brand['subdomain'],
            brand['brand_url'],
            brand['help_center_state'],
            brand['active'],
            brand['host_mapping']])

    click.echo(tabulate(table, headers=headers))



import csv
import click
import simplejson as json
from ..utilities import get_all_triggers, parse_actions_for_csv, parse_conditions_for_csv


@click.command()
@click.option('--directory', type=click.Path(exists=True, file_okay=False, writable=True, resolve_path=True),
              default='.')
@click.option('--filename', type=click.STRING, default='triggers')
@click.option('--format', type=click.Choice(['json', 'csv'], case_sensitive=False), default='json')
@click.option('--category_id', type=click.INT)
@click.option('--active_only', is_flag=True)
@click.pass_context
def get_triggers(ctx, directory, filename, format, category_id, active_only):
    """Get all triggers and save to file."""
    if ctx.obj['configuration'] == {}:
        raise click.UsageError('No configuration found. Try `zenkly configure`', ctx=ctx)

    triggers = get_all_triggers(config=ctx.obj['configuration'], category_id=category_id, active_only=active_only)
    path = '%s/%s.%s' % (directory, filename, format)

    with open(path, 'w') as outfile:
        if format.lower() == 'csv':
            fieldnames = set()
            formatted_triggers = []

            # format each trigger and append to list of formatted triggers
            for trigger in triggers:
                parsed_actions = parse_actions_for_csv(trigger['actions'])
                trigger.pop('actions', None)  # remove old actions key
                trigger = {**trigger, **parsed_actions}  # combine the trigger with parsed actions

                parsed_conditions = parse_conditions_for_csv(trigger['conditions'])
                trigger.pop('conditions', None)
                trigger = {**trigger, **parsed_conditions}

                fieldnames.update(trigger.keys(), parsed_actions.keys(), parsed_conditions.keys())
                fieldnames.update()

                formatted_triggers.append(trigger)

            fieldnames = sorted(fieldnames, reverse=True)  # put all the 'action:' columns at the end

            # move id and title to first and second columns, respectively
            old_id_index = fieldnames.index('id')
            old_title_index = fieldnames.index('title')
            fieldnames.insert(0, fieldnames.pop(old_id_index))
            fieldnames.insert(1, fieldnames.pop(old_title_index))

            writer = csv.DictWriter(outfile, fieldnames=fieldnames)
            writer.writeheader()

            for trigger in formatted_triggers:
                writer.writerow(trigger)
        else:
            json.dump({'triggers': triggers}, outfile, indent=2)

    click.echo('Triggers saved to %s' % path)



import os
import click
import simplejson as json
from ..utilities import post_all_macros


@click.command()
@click.option('--directory', type=click.Path(exists=True, file_okay=False, writable=True, resolve_path=True),
              default='.', prompt=True)
@click.option('--filename', type=click.STRING, required=True, prompt=True)
@click.pass_context
def add_macros(ctx, directory, filename):
    """Create macros from file."""
    if ctx.obj['configuration'] == {}:
        raise click.UsageError('No configuration found. Try `zenkly configure`', ctx=ctx)

    path = '%s/%s' % (directory, filename)

    if not os.path.exists(path):
        raise click.FileError(path, hint='File does not exist')

    with open(path, 'r') as infile:
        try:
            data = json.load(infile)
        except ValueError as e:
            raise click.UsageError('There was a problem loading %s: %s' % (path, e))

    if 'macros' not in data:
        raise click.UsageError('Missing `macros` key in %s' % path)

    if not type(data['macros']) is list:
        raise click.UsageError('Key `macros` in %s must be a list' % path)

    click.confirm('Are you sure you want to add %d macros?' % len(data['macros']), abort=True)

    post_all_macros(config=ctx.obj['configuration'], data=data['macros'])



import csv
import click
import simplejson as json
from ..utilities import get_all_macros, parse_actions_for_csv


@click.command()
@click.option('--directory', type=click.Path(exists=True, file_okay=False, writable=True, resolve_path=True),
              default='.')
@click.option('--filename', type=click.STRING, default='macros')
@click.option('--format', type=click.Choice(['json', 'csv'], case_sensitive=False), default='json')
@click.option('--category', type=click.STRING)
@click.option('--active_only', is_flag=True)
@click.pass_context
def get_macros(ctx, directory, filename, format, category, active_only):
    """Get all macros and save to file."""
    if ctx.obj['configuration'] == {}:
        raise click.UsageError('No configuration found. Try `zenkly configure`', ctx=ctx)

    macros = get_all_macros(config=ctx.obj['configuration'], category=category, active_only=active_only)
    path = '%s/%s.%s' % (directory, filename, format)

    with open(path, 'w') as outfile:
        if format.lower() == 'csv':
            fieldnames = set()
            formatted_macros = []

            # format each macro and append to list of formatted macros
            for macro in macros:
                parsed_actions = parse_actions_for_csv(macro['actions'])
                macro.pop('actions', None)  # remove old actions key
                macro = {**macro, **parsed_actions}  # combine the macro with parsed actions

                # add the macro category
                macro_category = macro['title'].split('::')[0]
                macro['macro_category'] = macro_category

                fieldnames.update(macro.keys(), parsed_actions.keys())
                formatted_macros.append(macro)

            fieldnames = sorted(fieldnames, reverse=True)  # put all the 'action:' columns at the end

            # move id and title to first and second columns, respectively
            old_id_index = fieldnames.index('id')
            old_title_index = fieldnames.index('title')
            fieldnames.insert(0, fieldnames.pop(old_id_index))
            fieldnames.insert(1, fieldnames.pop(old_title_index))

            writer = csv.DictWriter(outfile, fieldnames=fieldnames)
            writer.writeheader()

            for macro in formatted_macros:
                writer.writerow(macro)
        else:
            json.dump({'macros': macros}, outfile, indent=2)

    click.echo('Macros saved to %s' % path)



import os
from time import time
from pathlib import Path
import click
from ..utilities import get_all_hc_by_type, write_json, archive_directory, push_archive_to_remote


@click.command()
@click.option('--directory', type=click.Path(exists=True, file_okay=False, writable=True, resolve_path=True),
              default=str(Path.home()))
@click.option('--backup-remotely', is_flag=True)
@click.option('--remote-name', type=click.STRING, default='origin')
@click.pass_context
def backup_guide(ctx, directory, backup_remotely, remote_name):
    """Backup Guide categories, sections and articles."""
    if ctx.obj['configuration'] == {}:
        raise click.UsageError('No configuration found. Try `zenkly configure`', ctx=ctx)

    backup_time = int(time())

    for t in ['articles', 'sections', 'categories']:
        try:
            data = get_all_hc_by_type(config=ctx.obj['configuration'], guide_type=t)
        except ValueError as err:
            raise click.ClickException(err)

        output_path = os.path.join(directory, 'backup_%s' % backup_time)
        filename = '%s.json' % t

        write_json(output_path=output_path, filename=filename, data=data)

    archive_path = archive_directory(output_path)

    if backup_remotely:
        push_archive_to_remote(repo_dir=directory, remote_name=remote_name, archive_path=archive_path,
                               backup_time=backup_time)

    click.secho('Done!', fg='green')



import csv
import click
import simplejson as json
from ..utilities import get_all_automations, parse_actions_for_csv, parse_conditions_for_csv


@click.command()
@click.option('--directory', type=click.Path(exists=True, file_okay=False, writable=True, resolve_path=True),
              default='.')
@click.option('--filename', type=click.STRING, default='triggers')
@click.option('--format', type=click.Choice(['json', 'csv'], case_sensitive=False), default='json')
@click.option('--active_only', is_flag=True)
@click.pass_context
def get_automations(ctx, directory, filename, format, active_only):
    """Get all automations and save to file."""
    if ctx.obj['configuration'] == {}:
        raise click.UsageError('No configuration found. Try `zenkly configure`', ctx=ctx)

    automations = get_all_automations(config=ctx.obj['configuration'], active_only=active_only)
    path = '%s/%s.%s' % (directory, filename, format)

    with open(path, 'w') as outfile:
        if format.lower() == 'csv':
            fieldnames = set()
            formatted_automations = []

            # format each trigger and append to list of formatted automations
            for automation in automations:
                parsed_actions = parse_actions_for_csv(automation['actions'])
                automation.pop('actions', None)  # remove old actions key
                automation = {**automation, **parsed_actions}  # combine the trigger with parsed actions

                parsed_conditions = parse_conditions_for_csv(automation['conditions'])
                automation.pop('conditions', None)
                automation = {**automation, **parsed_conditions}

                fieldnames.update(automation.keys(), parsed_actions.keys(), parsed_conditions.keys())
                fieldnames.update()

                formatted_automations.append(automation)

            fieldnames = sorted(fieldnames, reverse=True)  # put all the 'action:' columns at the end

            # move id and title to first and second columns, respectively
            old_id_index = fieldnames.index('id')
            old_title_index = fieldnames.index('title')
            fieldnames.insert(0, fieldnames.pop(old_id_index))
            fieldnames.insert(1, fieldnames.pop(old_title_index))

            writer = csv.DictWriter(outfile, fieldnames=fieldnames)
            writer.writeheader()

            for automation in formatted_automations:
                writer.writerow(automation)
        else:
            json.dump({'automations': automations}, outfile, indent=2)

    click.echo('Automations saved to %s' % path)



import os
from time import time
import json
import shutil
import click
from zenkly.scripts.utilities import write_json


@click.command()
@click.option('--old-backup-file', type=click.File(), required=True)
@click.option('--new-backup-file', type=click.File(), required=True)
@click.option('--directory', type=click.Path(exists=True, file_okay=False, writable=True, resolve_path=True),
              default='.')
@click.pass_context
def create_article_mapping(ctx, directory, old_backup_file, new_backup_file):
    """Generate a JSON object with mapping based on provided backup files."""
    mapping_time = int(time())

    mapping = {}

    old_data = json.load(old_backup_file)
    new_data = json.load(new_backup_file)

    for o in old_data:
        old_id = o['id']
        old_title = o['name']

        for n in new_data:
            new_id = n['id']
            new_title = n['name']

            if old_title == new_title:
                mapping[old_id] = str(new_id)

    filename = 'mapping_%s.json' % mapping_time

    write_json(output_path=directory, filename=filename, data=mapping)

    click.secho('Done!', fg='green')



import os
import errno
import json
import click
import requests
import time
import shutil
import threading
from functools import wraps
import git
from .constants import VALID_HC_TYPES


def rate_limited(max_per_second: int):
    """
    Rate-limits the decorated function locally, for one process.
    :param max_per_second: the number of requests allowed per second
    :return: the decorated function
    """
    lock = threading.Lock()
    min_interval = 1.0 / max_per_second

    def decorate(func):
        last_time_called = time.perf_counter()

        @wraps(func)
        def rate_limited_function(*args, **kwargs):
            lock.acquire()
            nonlocal last_time_called
            try:
                elapsed = time.perf_counter() - last_time_called
                left_to_wait = min_interval - elapsed
                if left_to_wait > 0:
                    time.sleep(left_to_wait)

                return func(*args, **kwargs)
            finally:
                last_time_called = time.perf_counter()
                lock.release()

        return rate_limited_function

    return decorate


@rate_limited(1)
def get(config, url, params={}):
    """
    GET the provided endpoint.
    :param config: context config
    :param url: the url to GET
    :return:
    """
    r = requests.get(
        url,
        params=params,
        auth=(config['email'], config['password'])
    )

    # Check for HTTP errors (4xx, 5xx).
    try:
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        raise click.ClickException(err)

    # Attempt to parse JSON. If valid JSON contains an error, raise it.
    # If JSON is invalid, raise the error.
    try:
        res = r.json()
        if 'error' in res:
            raise click.ClickException(res['error'])
    except ValueError as err:
        raise click.ClickException(err)

    return res


@rate_limited(1)
def put(config, url, data):
    """
    PUT data to the provided endpoint.
    :param config: context config
    :param url: the url to PUT data to
    :param data: the data to PUT
    :return:
    """
    r = requests.put(
        url,
        auth=(config['email'], config['password']),
        json=data
    )

    # Check for HTTP errors (4xx, 5xx).
    try:
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        raise click.ClickException(err)

    # Attempt to parse JSON. If valid JSON contains an error, raise it.
    # If JSON is invalid, raise the error.
    try:
        res = r.json()
        if 'error' in res:
            raise click.ClickException(res['error'])
    except ValueError as err:
        raise click.ClickException(err)

    return res


@rate_limited(1)
def post(config, url, data={}):
    """
    POST data to the provided endpoint.
    :param config: context config
    :param url: the url to POST the data to
    :param data: the data to POST
    :return:
    """
    r = requests.post(
        url,
        auth=(config['email'], config['password']),
        json=data,
    )

    # Check for HTTP errors (4xx, 5xx).
    try:
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        raise click.ClickException(err)

    # Attempt to parse JSON. If valid JSON contains an error, raise it.
    # If JSON is invalid, raise the error.
    try:
        res = r.json()
        if 'error' in res:
            raise click.ClickException(res['error'])
    except ValueError as err:
        raise click.ClickException(err)

    return res


@rate_limited(1)
def post_form_data(config, url, data={}, files={}):
    """
    POST data to the provided endpoint.
    :param config: context config
    :param url: the url to POST the data to
    :param data: the data to POST
    :return:
    """

    r = requests.post(
        url,
        auth=(config['email'], config['password']),
        data=data,
        files=files
    )

    # Check for HTTP errors (4xx, 5xx).
    try:
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        raise click.ClickException(err)

    return r


def post_theme_import_job(config, brand_id):
    """
    Create help center theme import job.
    :param config: context config
    :param brand_id: the brand id for the relevant help center
    :return: job json
    """
    url = f"https://{config['subdomain']}.zendesk.com/api/v2/guide/theming/jobs/themes/imports"

    data = {
        'job': {
            'attributes': {
                'brand_id': brand_id,
                'format': 'zip',
            }
        }
    }

    res = post(config, url, data)

    return res['job']


def get_theme_job(config, job_id):
    """
    Retrieve job data (for polling)
    :param config: context config
    :param job_id: the job id to retrieve
    :return: job json
    """
    url = f"https://{config['subdomain']}.zendesk.com/api/v2/guide/theming/jobs/{job_id}"

    res = get(config, url)

    return res['job']

def post_theme(config, storage_url, parameters, files):
    """
    POST the theme zip file to the provided storage url
    :param config: context config
    :param storage_url: the storage url provided by import job response
    :param parameters: the parameters provided by the import job response
    :param files: theme zip file data
    :return:
    """
    data = {**parameters}

    res = post_form_data(config, storage_url, data=data, files=files)

    return res


def get_all_themes(config, brand_id):
    """
    Get all themes for the given brand id.
    :param config: context config
    :param brand_id: the brand id for the relevant help center
    :return list: list of all themes
    """
    url = f"https://{config['subdomain']}.zendesk.com/api/guide/theming/{brand_id}/themes.json"
    res = get(config, url)

    return res['themes']


def publish_theme(config, brand_id, theme_id):
    """
    Publish the given theme for the given brand.
    :param config: context config
    :param brand_id: the brand id
    :param theme_id: the theme id
    :return:
    """
    url = f"https://{config['subdomain']}.zendesk.com/api/guide/theming/{brand_id}/themes/{theme_id}/publish.json"
    res = post(config, url)

    return res['theme']


def get_all_macros(config, category=None, active_only=False):
    """
    Get all pages of macros from Zendesk.
    :param config: context config
    :param category: only get macros from this category
    :param active: flag to only include active macros
    :return:
    """
    url = f"https://{config['subdomain']}.zendesk.com/api/v2/macros.json"
    params = {}

    if active_only:
        params['active'] = 'true'

    if category:
        params['category'] = category

    res = get(config, url, params=params)
    all = res['macros']

    with click.progressbar(length=res['count'], label='Getting macros...') as bar:
        bar.update(len(res['macros']))

        while res['next_page']:
            res = get(config, res['next_page'], params=params)
            all = all + res['macros']
            bar.update(len(res['macros']))

    return all


def get_all_triggers(config, category_id=None, active_only=False):
    """
    Get all pages of triggers from Zendesk.
    :param config: context config
    :param category_id: only get triggers from this category
    :param active: flag to only include active triggers
    :return:
    """
    url = f"https://{config['subdomain']}.zendesk.com/api/v2/triggers.json"
    params = {}

    if active_only:
        params['active'] = 'true'

    if category_id:
        params['category_id'] = category_id

    res = get(config, url, params=params)
    all = res['triggers']

    with click.progressbar(length=res['count'], label='Getting triggers...') as bar:
        bar.update(len(res['triggers']))

        while res['next_page']:
            res = get(config, res['next_page'], params=params)
            all = all + res['triggers']
            bar.update(len(res['triggers']))

    return all


def get_all_automations(config, active_only=False):
    """
    Get all pages of automations from Zendesk.
    :param config: context config
    :param active: flag to only include active automations
    :return:
    """
    url = f"https://{config['subdomain']}.zendesk.com/api/v2/automations.json"
    params = {}

    if active_only:
        params['active'] = 'true'

    res = get(config, url, params=params)
    all = res['automations']

    with click.progressbar(length=res['count'], label='Getting automations...') as bar:
        bar.update(len(res['automations']))

        while res['next_page']:
            res = get(config, res['next_page'], params=params)
            all = all + res['automations']
            bar.update(len(res['automations']))

    return all


def parse_actions_for_csv(actions):
    parsed_actions = {}

    for action in actions:
        action_field = action['field']
        action_value = action['value']

        key_name = f"action:{action_field}"

        if not key_name in parsed_actions:
            parsed_actions[key_name] = []

        parsed_actions[key_name].append(action_value)

    return parsed_actions


def parse_conditions_for_csv(conditions):
    parsed_conditions = {}

    for condition in conditions['all']:
        key_name = f"condition:all:{condition['field']}"

        if not key_name in parsed_conditions:
            parsed_conditions[key_name] = []

        parsed_conditions[key_name].append(f"{condition['operator']} {condition['value']}")

    for condition in conditions['any']:
        key_name = f"condition:any:{condition['field']}"

        if not key_name in parsed_conditions:
            parsed_conditions[key_name] = []

        parsed_conditions[key_name].append(f"{condition['operator']} {condition['value']}")

    return parsed_conditions


def post_all_macros(config, data):
    """
    Create macros in Zendesk.
    :param config: context config
    :param data: the macro data to POST
    """
    entries = (
        'title', 'active', 'actions', 'restriction',
        'description', 'attachments'
    )

    succeeded = []
    failed = []

    with click.progressbar(length=len(data), label='Adding macros...') as bar:
        for m in data:
            macro = {'macro': {k: m[k] for k in m if k in entries}}

            url = f"https://{config['subdomain']}.zendesk.com/api/v2/macros.json"

            try:
                res = post(config, url, macro)
                succeeded.append((m['id'], res['macro']['id']))
            except click.ClickException as err:
                failed.append((m['id'], err.message))  # Record failures

            bar.update(1)

        click.secho('\n\nAddition complete!')

        if succeeded:
            click.secho('\nThe following macros were added: ', fg='green', bold=True)
            for s in succeeded:
                click.secho(f"{s[0]} (new id: {s[1]})", fg='green')

        if failed:
            click.secho('\nThe following macros could not be added: ', fg='red', bold=True)
            for f in failed:
                click.secho(f"{f[0]} ({f[1]})", fg='red')


def put_all_macros(config, data):
    """
    Update macros in Zendesk.
    :param config: context config
    :param data: the macro data to PUT
    """
    entries = (
        'title', 'active', 'actions', 'restriction',
        'description', 'attachments'
    )

    succeeded = []
    failed = []

    with click.progressbar(length=len(data), label='Updating macros...') as bar:
        for m in data:
            macro = {'macro': {k: m[k] for k in m if k in entries}}

            url = f"https://{config['subdomain']}.zendesk.com/api/v2/macros/{m['id']}.json"

            try:
                put(config, url, macro)
                succeeded.append(m['id'])
            except click.ClickException as err:
                failed.append((m['id'], err.message))  # Record failures

            bar.update(1)

        click.secho('\n\nUpdate complete!')

        if succeeded:
            click.secho('\nThe following macros were updated: ', fg='green', bold=True)
            for s in succeeded:
                click.secho(s, fg='green')

        if failed:
            click.secho('\nThe following macros could not be updated: ', fg='red', bold=True)
            for f in failed:
                click.secho(f"{f[0]} ({f[1]})", fg='red')


def get_all_locales(config):
    url = f"https://{config['subdomain']}.zendesk.com/api/v2/locales.json"
    res = get(config, url)
    all_locales = res['locales']

    with click.progressbar(length=res['count'], label='Getting locales...') as bar:
        bar.update(len(res['locales']))

        while res['next_page']:
            res = get(config, res['next_page'])
            all_locales.append(res['locales'])
            bar.update(len(res['locales']))

    return all_locales


def get_all_hc_by_type(config, guide_type):
    """
    Get all help center content by type (articles, sections, categories).
    :param config:
    :param guide_type:
    :return:
    """
    if guide_type not in VALID_HC_TYPES:
        raise ValueError(f"Type must be one of {VALID_HC_TYPES}")

    click.echo(f"Getting {guide_type} with translations...")

    all_data = []

    url = f"https://{config['subdomain']}.zendesk.com/api/v2/help_center/{guide_type}.json?include=translations"
    res = get(config, url)

    all_data = all_data + res[guide_type]

    with click.progressbar(length=res['count']) as bar:
        bar.update(len(res[guide_type]))

        while res['next_page']:  # Get all pages
            res = get(config, res['next_page'])
            all_data = all_data + res[guide_type]
            bar.update(len(res[guide_type]))

    return all_data


def get_all_brands(config):
    url = f"https://{config['subdomain']}.zendesk.com/api/v2/brands.json"
    res = get(config, url)

    return res['brands']


def confirm_or_create_path(path):
    # Check if path exists, and create it if it doesn't.
    if not os.path.exists(path):
        try:
            os.makedirs(path)
        except OSError as error:
            if error.errno != errno.EEXIST:
                raise


def write_json(output_path, filename, data):
    destination = os.path.join(output_path, filename)

    click.echo(f"Writing data to {click.format_filename(destination)}")

    confirm_or_create_path(output_path)

    with click.open_file(destination, 'w') as f:
        json.dump(data, f, indent=4)


def archive_directory(path):
    click.echo(f"Archiving directory: {click.format_filename(path, shorten=True)}")
    archive_name = shutil.make_archive(path, 'zip', path)  # Zip up the directory at path
    shutil.rmtree(path)  # Delete unzipped directory

    return archive_name


def push_archive_to_remote(repo_dir, remote_name, archive_path, backup_time):
    click.echo(f"Finding repository at {click.format_filename(repo_dir)}")
    repo = git.Repo(repo_dir)

    click.echo(f"Staging {archive_path}")
    repo.index.add([archive_path])

    commit_msg = f"Add backup @ {backup_time}"
    click.echo(f"Committing with message: {commit_msg}")
    repo.index.commit(commit_msg)

    click.echo(f"Pushing to remote {remote_name}")
    origin = repo.remote(remote_name)
    origin.push()

